from logger import logger
import lastversion
import download
class Get:
    def __init__(self, repo):
        self.repo = repo
        logger.debug('repo set to %(repo)s')

    def version(self):
        try:
            version = lastversion.latest(self.repo)
            assert version is not None
            return version
        except AssertionError:
            logger.debug('repo (%(repo)s) not found?')







import asyncio
import os
from pathlib import Path

import aiohttp
from gwss.logger import logger

async def download_file(url, package: str, name: str, version: str, ext: str, dest_dir: os.PathLike):
    """

    :param url:
    :param package: name of whole package
    :param name: name of package file
    :param version: version of package file
    :param ext: extension
    :param dest_dir: destination of files downloaded
    :return:
    """
    computed_filename = "%(name)s-%(version)s.%(ext)s".format(name, version, ext)
    assert Path(dest_dir).exists()

    dest_dir = Path(dest_dir)
    dest_file = Path.joinpath(dest_dir,computed_filename )
    logger.debug("Downloading file %(computed_filename)s to %(dest_file)s")
    async with aiohttp.ClientSession() as session:
        async with session.get(url) as response:
            if "content-disposition" in response.headers:
                header = response.headers["content-disposition"]
                file = header.split("filename=")[1]
            else:
                file = url.split("/")[-1]
            with open(file, mode="wb") as file:
                while True:
                    chunk = await response.content.read()
                    if not chunk:
                        break
                    file.write(chunk)
                print(f"Downloaded file {file}")


import logging
import sys

log = logging.getLogger('GWSS')

sout_handler = logging.StreamHandler(sys.stdout)
sout_handler.setLevel(logging.DEBUG)

sout_formatter = logging.Formatter("%(asctime)s:%(name)s:%(levelname)s:%(message)s")
sout_handler.setFormatter(sout_formatter)
log.addHandler(sout_handler)
logger = log



import os
import yaml
import pathlib
from pathlib import Path
from gwss.logger import logger


logger.debug("Creating and loading 'config' object")
# >>> Path.joinpath(Path.home(), '.gwss')
# PosixPath('/home/ken/.gwss')
config = yaml.safe_load(open(Path.joinpath(Path.home(), '.gwss'), 'r'))
logger.debug("Created 'config' object")



# package:
#  dir: /path/to/SorS/
#  scripts:
#    - abbreviated-name: path/to/javascript/file # without extension, will be guessed based on scripts vs styles
projects = {
    'bootstrap': {
        'dir': 'dist',
        'scripts': {
            'bootstrap': 'js/bootstrap.min.js',
            'bootstrap-bundle': 'js/bootstrap.bundle.min.js'
        },
        'styles': {
            'bootstrap-min': 'css/bootstrap.min.css',
            'bootstrap-grid': 'css/bootstrap-grid.min.css',
            'bootstrap-reboot': 'css/bootstrap-reboot.min.css',
            'bootstrap-utilities': 'css/bootstrap-utilities.min.css'
        }
    },
    'htmx': {
        'dir': '.',
        'scripts': {
            'htmx': 'htmx.js'
        }
    },
    'jquery': {
        'dir': 'dist',
        'scripts': {
            'jquery': 'jquery.min.js'
        }
    },
    'react': {
        'dir': 'umd',
        'scripts': {
            'react': 'react.production.min.js'
        }
    },
    'tailwindcss': {
        'dir': 'dist',
        'styles': {
            'tailwind': 'tailwind.css'
        }
    }
}



from importlib.metadata import version
import sys
import os
from pathlib import Path
from pprint import pprint

import click

from gwss import gwss, resolver_config
from gwss.config import config
from gwss.resolver import resolve_pkg
from gwss.utilities import prepare_config, path_validation, squish_info


class Site(object):
    def __init__(self, destination_directory):
        self.dest_dir = os.path.abspath(destination_directory or '.')

def print_version(ctx, param, value):
    if not value or ctx.resilient_parsing:
        return
    click.echo("v{}".format(version('gwss')))
    ctx.exit()

@click.group(invoke_without_command=True)
@click.option('--version', '-v', is_flag=True, callback=print_version,
              expose_value=False, is_eager=False)

@click.pass_context
def cli(ctx):
    pass

@cli.command(name='list', help="list the script and style names from the '{}' file".format(Path.joinpath(Path.home(), '.gwss')))
def ls():
    cfg = prepare_config(config)
    print(cfg)

@cli.command(help="get all urls and destination directories for styles and scripts")
@click.option('--dest-dir', '-d', default='.site')
def resolve(dest_dir):
    resolve_cfg = resolver_config.projects
    extension = ''
    for k, v in config.items():
        print("{}".format(v),)
        resolve_pkg(v)
    resolved_rendered = ''
    click.echo(resolved_rendered)

@cli.command()
@click.option('--dest-dir', '-d', default='.site')
def download(dest_dir):
    if path_validation(dest_dir):
        pass





from gwss.download import download_file
import resolver

class Unpkg:

    base_url = 'https://unpkg.com'

    def __init__(self, package, version, file, s_or_s):
        """
        Make sure every filename is correct before going through the entirety
        of the function
        :param package:
        :param version:
        :param file:
        :param s_or_s:
        """
        self.package = package
        self.version = version
        self.file = file
        self.s_or_s = s_or_s

        self.url = self.base_url + "%()s"


    def unpkg_dl(self, dest_dir):
        download_file(self.url, self.package, self.file, self.version, self.s_or_s, dest_dir)



#
# UNPKG Resolver
#
# return all data needed to download scripts and styles from UNPKG

from pprint import pprint
import yaml
import pathlib
from pathlib import Path

def resolve_pkg(package):
    current_directory = Path(__file__).parent
    pprint(current_directory)



from setuptools import setup

setup(
    name='gwss',
    version='0.0.1',

    url='',
    scripts=['bin/gwss'],
    license='MIT',
    author='Ken Spencer / IotaSpencer',
    author_email='me@iotaspencer.me',
    description='get web scripts & styles',
    long_description='get web scripts and styles',
    packages=['gwss'],
    install_requires=[
        'lastversion~=3.4.6',
        'aiohttp~=3.9.2',
        'click~=8.1.7',
        'pytest~=8.0.0',
        'setuptools~=59.6.0'
    ],
    classifiers= [
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Development Status :: 4 - Beta",
        "Environment :: Web Environment",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "Programming Language :: Python :: 3.13",
        "Topic :: Internet :: WWW/HTTP :: Site Management",
        "Typing :: Typed",
        "Topic :: System :: Software Distribution"
    ]
)



from __future__ import annotations

import os
from pathlib import Path
import pathlib
from pprint import pprint

from jinja2 import Environment, BaseLoader
from colorama import Fore, Back, Style


ls_tmpl = """
{%- for type, values in config.items() %}
    {{- Style.BRIGHT }}{{- type.upper() }}{{ Style.NORMAL }}
    {% for value in values %}
    {{ value }}
{% endfor -%}
{%- endfor -%}
"""

def prepare_config(config: dict) -> str:
    """

    :param config:
    :return: str
    """
    rtmpl = Environment(keep_trailing_newline=False, trim_blocks=True, lstrip_blocks=True, loader=BaseLoader()).from_string(ls_tmpl)
    return rtmpl.render(config=config, Style=Style, Fore=Fore, Back=Back)


def squish_info(config) -> dict:
    info = config
    info_dict = {}
    pprint(config)
    for k, v, in info.items():
        info_dict[k] = {}
        for k2 in v:
            pprint(k2)
            info_dict[k][k2] = {}

    return info_dict

    # check if path exists
def path_validation(path: str | os.PathLike | Path) -> bool:
    """

    :param path:
    :return: True
    """
    return Path.exists(path, follow_symlinks=True)

# if it doesn't, create it
def path_create(path: str | os.PathLike | Path) -> bool:
    """

    :param path:
    :return: True
    """
    if path_validation(path):
        return False
    else:
        Path.mkdir(path, parents=True, exist_ok=True)
